@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 """
 Custom handlers may be created to handle other objects. Each custom handler
 must derive from :class:`jsonpickle.handlers.BaseHandler` and
@@ -37,23 +39,59 @@ class Registry(object):
 
     def __init__(self):
         self._handlers = {}
+        self._base_handlers = {}
 
-    def register(self, cls, handler):
+    def get(self, cls_or_name, default=None):
+        """
+        :param cls_or_name: the type or its fully qualified name
+        :param default: default value, if a matching handler is not found
+
+        Looks up a handler by type reference or its fully qualified name. If a direct match
+        is not found, the search is performed over all handlers registered with base=True.
+        """
+        handler = self._handlers.get(cls_or_name)
+        if handler is None and util.is_type(cls_or_name):  # attempt to find a base class
+            for cls, base_handler in self._base_handlers.items():
+                if issubclass(cls_or_name, cls):
+                    return base_handler
+        return default if handler is None else handler
+
+    def register(self, cls, handler=None, base=False):
         """Register the a custom handler for a class
 
         :param cls: The custom object class to handle
-        :param handler: The custom handler class
+        :param handler: The custom handler class (if None, a decorator wrapper is returned)
+        :param base: Indicates whether the handler should be registered for all subclasses
 
+        This function can be also used as a decorator by omitting the `handler` argument:
+
+        @jsonpickle.handlers.register(Foo, base=True)
+        class FooHandler(jsonpickle.handlers.BaseHandler):
+            pass
         """
-        key = util.importable_name(cls)
-        self._handlers[key] = handler
+        if handler is None:
+            def _register(handler_cls):
+                self.register(cls, handler=handler_cls, base=base)
+                return handler_cls
+            return _register
+        if not util.is_type(cls):
+            raise TypeError('{0!r} is not a class/type'.format(cls))
+        # store both the name and the actual type for the ugly cases like
+        # _sre.SRE_Pattern that cannot be loaded back directly
+        self._handlers[util.importable_name(cls)] = self._handlers[cls] = handler
+        if base:
+            # only store the actual type for subclass checking
+            self._base_handlers[cls] = handler
 
-    def get(self, class_name):
-        return self._handlers.get(class_name)
+    def unregister(self, cls):
+        self._handlers.pop(cls, None)
+        self._handlers.pop(util.importable_name(cls), None)
+        self._base_handlers.pop(cls, None)
 
 
 registry = Registry()
 register = registry.register
+unregister = registry.unregister
 get = registry.get
 
 
@@ -70,12 +108,22 @@ class BaseHandler(object):
         self.context = context
 
     def flatten(self, obj, data):
-        """Flatten `obj` into a json-friendly form and write result in `data`"""
+        """
+        Flatten `obj` into a json-friendly form and write result to `data`.
+
+        :param object obj: The object to be serialized.
+        :param dict data: A partially filled dictionary which will contain the
+            json-friendly representation of `obj` once this method has
+            finished.
+        """
         raise NotImplementedError('You must implement flatten() in %s' %
                                   self.__class__)
 
     def restore(self, obj):
-        """Restore the json-friendly `obj` to the registered type"""
+        """
+        Restore an object of the registered type from the json-friendly
+        representation `obj` and return it.
+        """
         raise NotImplementedError('You must implement restore() in %s' %
                                   self.__class__)
 
