@@ -13,6 +13,8 @@ import unittest
 import collections
 
 import jsonpickle
+import jsonpickle.backend
+import jsonpickle.handlers
 
 from jsonpickle import tags, util
 from jsonpickle.compat import unicode
@@ -73,7 +75,6 @@ class ThingWithProps(object):
         return self.identity == other.identity
 
 
-
 class PicklingTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -136,11 +137,11 @@ class PicklingTestCase(unittest.TestCase):
         for s in setlist:
             self.assertTrue(s in flattened[tags.SET])
 
-        setA_pickled = {tags.SET: setlist}
-        self.assertEqual(setA, self.unpickler.restore(setA_pickled))
+        setA_pickle = {tags.SET: setlist}
+        self.assertEqual(setA, self.unpickler.restore(setA_pickle))
 
     def test_dict(self):
-        dictA = {'key1': 1.0, 'key2': 20, 'key3': 'thirty'}
+        dictA = {'key1': 1.0, 'key2': 20, 'key3': 'thirty', tags.JSON_KEY + '6': 6}
         self.assertEqual(dictA, self.pickler.flatten(dictA))
         self.assertEqual(dictA, self.unpickler.restore(dictA))
         dictB = {}
@@ -150,26 +151,26 @@ class PicklingTestCase(unittest.TestCase):
     def test_tuple(self):
         # currently all collections are converted to lists
         tupleA = (4, 16, 32)
-        tupleA_pickled = {tags.TUPLE: [4, 16, 32]}
-        self.assertEqual(tupleA_pickled, self.pickler.flatten(tupleA))
-        self.assertEqual(tupleA, self.unpickler.restore(tupleA_pickled))
+        tupleA_pickle = {tags.TUPLE: [4, 16, 32]}
+        self.assertEqual(tupleA_pickle, self.pickler.flatten(tupleA))
+        self.assertEqual(tupleA, self.unpickler.restore(tupleA_pickle))
         tupleB = (4,)
-        tupleB_pickled = {tags.TUPLE: [4]}
-        self.assertEqual(tupleB_pickled, self.pickler.flatten(tupleB))
-        self.assertEqual(tupleB, self.unpickler.restore(tupleB_pickled))
+        tupleB_pickle = {tags.TUPLE: [4]}
+        self.assertEqual(tupleB_pickle, self.pickler.flatten(tupleB))
+        self.assertEqual(tupleB, self.unpickler.restore(tupleB_pickle))
 
     def test_tuple_roundtrip(self):
-        data = (1,2,3)
+        data = (1, 2, 3)
         newdata = jsonpickle.decode(jsonpickle.encode(data))
         self.assertEqual(data, newdata)
 
     def test_set_roundtrip(self):
-        data = set([1,2,3])
+        data = set([1, 2, 3])
         newdata = jsonpickle.decode(jsonpickle.encode(data))
         self.assertEqual(data, newdata)
 
     def test_list_roundtrip(self):
-        data = [1,2,3]
+        data = [1, 2, 3]
         newdata = jsonpickle.decode(jsonpickle.encode(data))
         self.assertEqual(data, newdata)
 
@@ -204,7 +205,7 @@ class PicklingTestCase(unittest.TestCase):
         self.assertEqual('a string', inflated[2])
 
     def test_classdict(self):
-        dict = {'k1':Thing('one'), 'k2':Thing('two'), 'k3':3}
+        dict = {'k1': Thing('one'), 'k2': Thing('two'), 'k3': 3}
 
         flattened = self.pickler.flatten(dict)
         self.assertEqual('one', flattened['k1']['name'])
@@ -298,9 +299,8 @@ class PicklingTestCase(unittest.TestCase):
         obj.typeref = object
 
         flattened = self.pickler.flatten(obj)
-        self.assertEqual(flattened['typeref'], {
-                            tags.TYPE: '__builtin__.object',
-                         })
+        self.assertEqual(flattened['typeref'],
+                         {tags.TYPE: '__builtin__.object'})
 
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(inflated.typeref, object)
@@ -314,9 +314,8 @@ class PicklingTestCase(unittest.TestCase):
         obj.classref = Thing
 
         flattened = self.pickler.flatten(obj)
-        self.assertEqual(flattened['classref'], {
-                            tags.TYPE: 'jsonpickle_test.Thing',
-                         })
+        self.assertEqual(flattened['classref'],
+                         {tags.TYPE: 'jsonpickle_test.Thing'})
 
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(inflated.classref, Thing)
@@ -377,134 +376,163 @@ class JSONPickleTestCase(unittest.TestCase):
     def setUp(self):
         self.obj = Thing('A name')
         self.expected_json = (
-                '{"'+tags.OBJECT+'": "jsonpickle_test.Thing",'
-                ' "name": "A name", "child": null}')
+            '{"%s": "jsonpickle_test.Thing", "name": "A name", "child": null}'
+            % tags.OBJECT
+        )
 
     def test_encode(self):
         expect = self.obj
-        pickled = jsonpickle.encode(self.obj)
-        actual = jsonpickle.decode(pickled)
+        pickle = jsonpickle.encode(self.obj)
+        actual = jsonpickle.decode(pickle)
         self.assertEqual(expect.name, actual.name)
         self.assertEqual(expect.child, actual.child)
 
     def test_encode_notunpicklable(self):
         expect = {'name': 'A name', 'child': None}
-        pickled = jsonpickle.encode(self.obj, unpicklable=False)
-        actual = jsonpickle.decode(pickled)
+        pickle = jsonpickle.encode(self.obj, unpicklable=False)
+        actual = jsonpickle.decode(pickle)
         self.assertEqual(expect['name'], actual['name'])
 
     def test_decode(self):
-        unpickled = jsonpickle.decode(self.expected_json)
-        self.assertEqual(self.obj.name, unpickled.name)
-        self.assertEqual(type(self.obj), type(unpickled))
+        actual = jsonpickle.decode(self.expected_json)
+        self.assertEqual(self.obj.name, actual.name)
+        self.assertEqual(type(self.obj), type(actual))
 
     def test_json(self):
         expect = self.obj
-        pickled = jsonpickle.encode(self.obj)
-        actual = jsonpickle.decode(pickled)
+        pickle = jsonpickle.encode(self.obj)
+        actual = jsonpickle.decode(pickle)
         self.assertEqual(actual.name, expect.name)
         self.assertEqual(actual.child, expect.child)
 
-        unpickled = jsonpickle.decode(self.expected_json)
-        self.assertEqual(self.obj.name, unpickled.name)
-        self.assertEqual(type(self.obj), type(unpickled))
+        actual = jsonpickle.decode(self.expected_json)
+        self.assertEqual(self.obj.name, actual.name)
+        self.assertEqual(type(self.obj), type(actual))
 
     def test_unicode_dict_keys(self):
         uni = unichr(0x1234)
-        pickled = jsonpickle.encode({uni: uni})
-        unpickled = jsonpickle.decode(pickled)
-        self.assertTrue(uni in unpickled)
-        self.assertEqual(unpickled[uni], uni)
+        pickle = jsonpickle.encode({uni: uni})
+        actual = jsonpickle.decode(pickle)
+        self.assertTrue(uni in actual)
+        self.assertEqual(actual[uni], uni)
 
     def test_tuple_dict_keys_default(self):
         """Test that we handle dictionaries with tuples as keys."""
-        tuple_dict = {(1, 2): 3, (4, 5): { (7, 8): 9 }}
-        pickled = jsonpickle.encode(tuple_dict)
+        tuple_dict = {(1, 2): 3, (4, 5): {(7, 8): 9}}
+        pickle = jsonpickle.encode(tuple_dict)
         expect = {'(1, 2)': 3, '(4, 5)': {'(7, 8)': 9}}
-        actual = jsonpickle.decode(pickled)
+        actual = jsonpickle.decode(pickle)
         self.assertEqual(expect, actual)
 
         tuple_dict = {(1, 2): [1, 2]}
-        pickled = jsonpickle.encode(tuple_dict)
-        unpickled = jsonpickle.decode(pickled)
-        self.assertEqual(unpickled['(1, 2)'], [1, 2])
+        pickle = jsonpickle.encode(tuple_dict)
+        actual = jsonpickle.decode(pickle)
+        self.assertEqual(actual['(1, 2)'], [1, 2])
 
     def test_tuple_dict_keys_with_keys_enabled(self):
         """Test that we handle dictionaries with tuples as keys."""
-        tuple_dict = {(1, 2): 3, (4, 5): { (7, 8): 9 }}
-        pickled = jsonpickle.encode(tuple_dict, keys=True)
+        tuple_dict = {(1, 2): 3, (4, 5): {(7, 8): 9}}
+        pickle = jsonpickle.encode(tuple_dict, keys=True)
         expect = tuple_dict
-        actual = jsonpickle.decode(pickled, keys=True)
+        actual = jsonpickle.decode(pickle, keys=True)
         self.assertEqual(expect, actual)
 
         tuple_dict = {(1, 2): [1, 2]}
-        pickled = jsonpickle.encode(tuple_dict, keys=True)
-        unpickled = jsonpickle.decode(pickled, keys=True)
-        self.assertEqual(unpickled[(1, 2)], [1, 2])
+        pickle = jsonpickle.encode(tuple_dict, keys=True)
+        actual = jsonpickle.decode(pickle, keys=True)
+        self.assertEqual(actual[(1, 2)], [1, 2])
+
+    def test_None_dict_key_default(self):
+        # We do string coercion for non-string keys so None becomes 'None'
+        expect = {'null': None}
+        obj = {None: None}
+        pickle = jsonpickle.encode(obj)
+        actual = jsonpickle.decode(pickle)
+        self.assertEqual(expect, actual)
+
+    def test_None_dict_key_with_keys_enabled(self):
+        expect = {None: None}
+        obj = {None: None}
+        pickle = jsonpickle.encode(obj, keys=True)
+        actual = jsonpickle.decode(pickle, keys=True)
+        self.assertEqual(expect, actual)
 
     def test_object_dict_keys(self):
         """Test that we handle random objects as keys.
 
         """
         thing = Thing('random')
-        pickled = jsonpickle.encode({thing: True})
-        unpickled = jsonpickle.decode(pickled)
-        self.assertEqual(unpickled, {unicode('Thing("random")'): True})
+        pickle = jsonpickle.encode({thing: True})
+        actual = jsonpickle.decode(pickle)
+        self.assertEqual(actual, {unicode('Thing("random")'): True})
 
     def test_int_dict_keys_defaults(self):
         int_dict = {1000: [1, 2]}
-        pickled = jsonpickle.encode(int_dict)
-        unpickled = jsonpickle.decode(pickled)
-        self.assertEqual(unpickled['1000'], [1, 2])
+        pickle = jsonpickle.encode(int_dict)
+        actual = jsonpickle.decode(pickle)
+        self.assertEqual(actual['1000'], [1, 2])
 
     def test_int_dict_keys_with_keys_enabled(self):
         int_dict = {1000: [1, 2]}
-        pickled = jsonpickle.encode(int_dict, keys=True)
+        pickle = jsonpickle.encode(int_dict, keys=True)
+        actual = jsonpickle.decode(pickle, keys=True)
+        self.assertEqual(actual[1000], [1, 2])
+
+    def test_string_key_requiring_escape_dict_keys_with_keys_enabled(self):
+        json_key_dict = {tags.JSON_KEY + '6': [1, 2]}
+        pickled = jsonpickle.encode(json_key_dict, keys=True)
         unpickled = jsonpickle.decode(pickled, keys=True)
-        self.assertEqual(unpickled[1000], [1, 2])
+        self.assertEqual(unpickled[tags.JSON_KEY + '6'], [1, 2])
+
+    def test_string_key_not_requiring_escape_dict_keys_with_keys_enabled(self):
+        """ test that string keys that does not require escaping are not escaped """
+        str_dict = {'name': [1, 2]}
+        pickled = jsonpickle.encode(str_dict, keys=True)
+        unpickled = jsonpickle.decode(pickled)
+        self.assertTrue('name' in unpickled)
 
     def test_list_of_objects(self):
         """Test that objects in lists are referenced correctly"""
         a = Thing('a')
         b = Thing('b')
-        pickled = jsonpickle.encode([a, b, b])
-        unpickled = jsonpickle.decode(pickled)
-        self.assertEqual(unpickled[1], unpickled[2])
-        self.assertEqual(type(unpickled[0]), Thing)
-        self.assertEqual(unpickled[0].name, 'a')
-        self.assertEqual(unpickled[1].name, 'b')
-        self.assertEqual(unpickled[2].name, 'b')
+        pickle = jsonpickle.encode([a, b, b])
+        actual = jsonpickle.decode(pickle)
+        self.assertEqual(actual[1], actual[2])
+        self.assertEqual(type(actual[0]), Thing)
+        self.assertEqual(actual[0].name, 'a')
+        self.assertEqual(actual[1].name, 'b')
+        self.assertEqual(actual[2].name, 'b')
 
     def test_refs_keys_values(self):
         """Test that objects in dict keys are referenced correctly
         """
         j = Thing('random')
         object_dict = {j: j}
-        pickled = jsonpickle.encode(object_dict, keys=True)
-        unpickled = jsonpickle.decode(pickled, keys=True)
-        self.assertEqual(list(unpickled.keys()), list(unpickled.values()))
+        pickle = jsonpickle.encode(object_dict, keys=True)
+        actual = jsonpickle.decode(pickle, keys=True)
+        self.assertEqual(list(actual.keys()), list(actual.values()))
 
     def test_object_keys_to_list(self):
         """Test that objects in dict values are referenced correctly
         """
         j = Thing('random')
         object_dict = {j: [j, j]}
-        pickled = jsonpickle.encode(object_dict, keys=True)
-        unpickled = jsonpickle.decode(pickled, keys=True)
-        obj = list(unpickled.keys())[0]
+        pickle = jsonpickle.encode(object_dict, keys=True)
+        actual = jsonpickle.decode(pickle, keys=True)
+        obj = list(actual.keys())[0]
         self.assertEqual(j.name, obj.name)
-        self.assertTrue(obj is unpickled[obj][0])
-        self.assertTrue(obj is unpickled[obj][1])
+        self.assertTrue(obj is actual[obj][0])
+        self.assertTrue(obj is actual[obj][1])
 
     def test_refs_in_objects(self):
         """Test that objects in lists are referenced correctly"""
         a = Thing('a')
         b = Thing('b')
-        pickled = jsonpickle.encode([a, b, b])
-        unpickled = jsonpickle.decode(pickled)
-        self.assertNotEqual(unpickled[0], unpickled[1])
-        self.assertEqual(unpickled[1], unpickled[2])
-        self.assertTrue(unpickled[1] is unpickled[2])
+        pickle = jsonpickle.encode([a, b, b])
+        actual = jsonpickle.decode(pickle)
+        self.assertNotEqual(actual[0], actual[1])
+        self.assertEqual(actual[1], actual[2])
+        self.assertTrue(actual[1] is actual[2])
 
     def test_refs_recursive(self):
         """Test that complicated recursive refs work"""
@@ -565,22 +593,26 @@ class JSONPickleTestCase(unittest.TestCase):
                 backend in jsonpickle.json._encoder_options or
                 backend in jsonpickle.json._decoder_exceptions)
 
-    def test_load_backend_skips_bad_encode(self):
+    def test_load_backend_handles_bad_encode(self):
         """Test that we ignore bad encoders"""
 
-        jsonpickle.load_backend('os.path', 'bad!', 'split', AttributeError)
+        load_backend = jsonpickle.load_backend
+        self.assertFalse(load_backend('os.path', 'bad!', 'split',
+                                      AttributeError))
         self.failIf(self._backend_is_partially_loaded('os.path'))
 
-    def test_load_backend_skips_bad_decode(self):
+    def test_load_backend_raises_on_bad_decode(self):
         """Test that we ignore bad decoders"""
 
-        jsonpickle.load_backend('os.path', 'join', 'bad!', AttributeError)
+        load_backend = jsonpickle.load_backend
+        self.assertFalse(load_backend('os.path', 'join', 'bad!', AttributeError))
         self.failIf(self._backend_is_partially_loaded('os.path'))
 
-    def test_load_backend_skips_bad_decoder_exceptions(self):
+    def test_load_backend_handles_bad_loads_exc(self):
         """Test that we ignore bad decoder exceptions"""
 
-        jsonpickle.load_backend('os.path', 'join', 'split', 'bad!')
+        load_backend = jsonpickle.load_backend
+        self.assertFalse(load_backend('os.path', 'join', 'split', 'bad!'))
         self.failIf(self._backend_is_partially_loaded('os.path'))
 
     def test_list_item_reference(self):
@@ -652,6 +684,26 @@ class PicklableNamedTuple(object):
         return instance
 
 
+class PicklableNamedTupleEx(object):
+    """
+    A picklable namedtuple wrapper, to demonstrate the need
+    for protocol 4 compatibility. Yes, this is contrived in
+    its use of new, but it demonstrates the issue.
+    """
+    def __getnewargs__(self):
+        raise NotImplementedError("This class needs __getnewargs_ex__")
+
+    def __new__(cls, newargs=__getnewargs__, **kwargs):
+        # it's necessary to use the correct class name for class resolution
+        # classes that fake their own names may never be unpicklable
+        ntuple = collections.namedtuple(cls.__name__, sorted(kwargs.keys()))
+        ntuple.__getnewargs_ex__ = (lambda self: ((), kwargs))
+        ntuple.__getnewargs__ = newargs
+        instance = ntuple.__new__(ntuple,
+                                  *[b for a, b in sorted(kwargs.items())])
+        return instance
+
+
 class PickleProtocol2Thing(object):
 
     def __init__(self, *args):
@@ -680,6 +732,7 @@ class PickleProtocol2Thing(object):
 slotmagic = PickleProtocol2Thing('slotmagic')
 dictmagic = PickleProtocol2Thing('dictmagic')
 
+
 class PickleProtocol2GetState(PickleProtocol2Thing):
     def __new__(cls, *args):
         instance = super(PickleProtocol2GetState, cls).__new__(cls)
@@ -689,14 +742,17 @@ class PickleProtocol2GetState(PickleProtocol2Thing):
     def __getstate__(self):
         return 'I am magic'
 
+
 class PickleProtocol2GetStateDict(PickleProtocol2Thing):
     def __getstate__(self):
         return {'magic': True}
 
+
 class PickleProtocol2GetStateSlots(PickleProtocol2Thing):
     def __getstate__(self):
         return (None, {'slotmagic': slotmagic})
 
+
 class PickleProtocol2GetStateSlotsDict(PickleProtocol2Thing):
     def __getstate__(self):
         return ({'dictmagic': dictmagic}, {'slotmagic': slotmagic})
@@ -746,7 +802,7 @@ class PickleProtocol2ReduceTuple(object):
                 ('yam', 1),  # args
                 None,  # state
                 iter([]),  # listitems
-                iter([]), # dictitems
+                iter([]),  # dictitems
                 )
 
 
@@ -758,12 +814,13 @@ class PickleProtocol2ReduceTupleFunc(object):
     def __init__(self, argval, optional=None):
         self.argval = argval
         self.optional = optional
+
     def __reduce__(self):
         return (protocol_2_reduce_tuple_func,  # callable
                 ('yam', 1),  # args
                 None,  # state
                 iter([]),  # listitems
-                iter([]), # dictitems
+                iter([]),  # dictitems
                 )
 
 
@@ -785,7 +842,7 @@ class PickleProtocol2ReduceNewobj(PickleProtocol2ReduceTupleFunc):
                 (PickleProtocol2ReduceNewobj, 'yam', 1),  # args
                 None,  # state
                 iter([]),  # listitems
-                iter([]), # dictitems
+                iter([]),  # dictitems
                 )
 
 
@@ -795,7 +852,7 @@ class PickleProtocol2ReduceTupleState(PickleProtocol2ReduceTuple):
                 ('yam', 1),  # args
                 {'foo': 1},  # state
                 iter([]),  # listitems
-                iter([]), # dictitems
+                iter([]),  # dictitems
                 )
 
 
@@ -808,7 +865,7 @@ class PickleProtocol2ReduceTupleSetState(PickleProtocol2ReduceTuple):
                 ('yam', 1),  # args
                 {'foo': 1},  # state
                 iter([]),  # listitems
-                iter([]), # dictitems
+                iter([]),  # dictitems
                 )
 
 
@@ -824,7 +881,7 @@ class PickleProtocol2ReduceTupleStateSlots(object):
                 ('yam', 1),  # args
                 {'foo': 1},  # state
                 iter([]),  # listitems
-                iter([]), # dictitems
+                iter([]),  # dictitems
                 )
 
 
@@ -837,12 +894,13 @@ class PickleProtocol2ReduceListitemsAppend(object):
                 (),  # args
                 {},  # state
                 iter(['foo', 'bar']),  # listitems
-                iter([]), # dictitems
+                iter([]),  # dictitems
                 )
 
     def append(self, item):
         self.inner.append(item)
 
+
 class PickleProtocol2ReduceListitemsExtend(object):
     def __init__(self):
         self.inner = []
@@ -852,12 +910,13 @@ class PickleProtocol2ReduceListitemsExtend(object):
                 (),  # args
                 {},  # state
                 iter(['foo', 'bar']),  # listitems
-                iter([]), # dictitems
+                iter([]),  # dictitems
                 )
 
     def extend(self, items):
         self.inner.exend(items)
 
+
 class PickleProtocol2ReduceDictitems(object):
     def __init__(self):
         self.inner = {}
@@ -867,12 +926,13 @@ class PickleProtocol2ReduceDictitems(object):
                 (),  # args
                 {},  # state
                 [],  # listitems
-                iter(zip(['foo', 'bar'],['foo', 'bar'])), # dictitems
+                iter(zip(['foo', 'bar'], ['foo', 'bar'])),  # dictitems
                 )
 
     def __setitem__(self, k, v):
         return self.inner.__setitem__(k, v)
 
+
 class PickleProtocol2Classic:
 
     def __init__(self, foo):
@@ -884,12 +944,62 @@ class PickleProtocol2ClassicInitargs:
     def __init__(self, foo, bar=None):
         self.foo = foo
         if bar:
-            self.bar=bar
+            self.bar = bar
 
     def __getinitargs__(self):
         return ('choo', 'choo')
 
 
+class PicklingProtocol4TestCase(unittest.TestCase):
+
+    def test_pickle_newargs_ex(self):
+        """
+        Ensure we can pickle and unpickle an object whose class needs arguments
+        to __new__ and get back the same typle
+        """
+        instance = PicklableNamedTupleEx(**{'a': 'b', 'n': 2})
+        encoded = jsonpickle.encode(instance)
+        decoded = jsonpickle.decode(encoded)
+        self.assertEqual(instance, decoded)
+
+    def test_validate_reconstruct_by_newargs_ex(self):
+        """
+        Ensure that the exemplar tuple's __getnewargs_ex__ works
+        This is necessary to know whether the breakage exists
+        in jsonpickle or not
+        """
+        instance = PicklableNamedTupleEx(**{'a': 'b', 'n': 2})
+        args, kwargs = instance.__getnewargs_ex__()
+        newinstance = PicklableNamedTupleEx.__new__(PicklableNamedTupleEx,
+                                                    *args, **kwargs)
+        self.assertEqual(instance, newinstance)
+
+    def test_references(self):
+        shared = Thing('shared')
+        instance = PicklableNamedTupleEx(**{'a': shared, 'n': shared})
+        child = Thing('child')
+        shared.child = child
+        child.child = instance
+
+        encoded = jsonpickle.encode(instance)
+        decoded = jsonpickle.decode(encoded)
+
+        self.assertEqual(decoded[0], decoded[1])
+        self.assertTrue(decoded[0] is decoded[1])
+        self.assertTrue(decoded.a is decoded.n)
+        self.assertEqual(decoded.a.name, 'shared')
+        self.assertEqual(decoded.a.child.name, 'child')
+        self.assertTrue(decoded.a.child.child is decoded)
+        self.assertTrue(decoded.n.child.child is decoded)
+        self.assertTrue(decoded.a.child is decoded.n.child)
+
+        self.assertEqual(decoded.__class__.__name__,
+                         PicklableNamedTupleEx.__name__)
+        # TODO the class itself looks just like the real class, but it's
+        # actually a reconstruction; PicklableNamedTupleEx is not type(decoded).
+        self.assertFalse(decoded.__class__ is PicklableNamedTupleEx)
+
+
 class PicklingProtocol2TestCase(unittest.TestCase):
 
     def test_classic_init_has_args(self):
@@ -918,6 +1028,18 @@ class PicklingProtocol2TestCase(unittest.TestCase):
         decoded = jsonpickle.decode(encoded)
         self.assertEqual(decoded.bar, 'choo')
 
+    def test_reduce_complex_num(self):
+        instance = 5j
+        encoded = jsonpickle.encode(instance)
+        decoded = jsonpickle.decode(encoded)
+        self.assertEqual(decoded, instance)
+
+    def test_reduce_complex_zero(self):
+        instance = 0j
+        encoded = jsonpickle.encode(instance)
+        decoded = jsonpickle.decode(encoded)
+        self.assertEqual(decoded, instance)
+
     def test_reduce_dictitems(self):
         'Test reduce with dictitems set (as a generator)'
         instance = PickleProtocol2ReduceDictitems()
@@ -940,9 +1062,8 @@ class PicklingProtocol2TestCase(unittest.TestCase):
         self.assertEqual(decoded.inner, ['foo', 'bar'])
 
     def test_reduce_state_setstate(self):
-        'Test reduce with the optional state argument set, on an object with a'\
-        '__setstate__'
-        # nosetests only shows first line of docstring
+        'Test reduce with the optional state argument set, on an object with '\
+            'a __setstate__'
 
         instance = PickleProtocol2ReduceTupleSetState(5)
         encoded = jsonpickle.encode(instance)
@@ -953,8 +1074,8 @@ class PicklingProtocol2TestCase(unittest.TestCase):
         self.assertFalse(hasattr(decoded, 'foo'))
 
     def test_reduce_state_no_dict(self):
-        'Test reduce with the optional state argument set, on an object with'\
-        'no __dict__, and no __setstate__'
+        'Test reduce with the optional state argument set, on an object with '\
+            'no __dict__, and no __setstate__'
 
         instance = PickleProtocol2ReduceTupleStateSlots(5)
         encoded = jsonpickle.encode(instance)
@@ -964,8 +1085,8 @@ class PicklingProtocol2TestCase(unittest.TestCase):
         self.assertEqual(decoded.foo, 1)
 
     def test_reduce_state_dict(self):
-        'Test reduce with the optional state argument set, on an object with a'\
-        '__dict__, and no __setstate__'
+        'Test reduce with the optional state argument set, on an object with '\
+            'a __dict__, and no __setstate__'
 
         instance = PickleProtocol2ReduceTupleState(5)
         encoded = jsonpickle.encode(instance)
@@ -1034,7 +1155,6 @@ class PicklingProtocol2TestCase(unittest.TestCase):
         decoded = jsonpickle.decode(encoded)
         self.assertEqual(decoded, slotmagic)
 
-
     def test_pickle_newargs(self):
         """
         Ensure we can pickle and unpickle an object whose class needs arguments
@@ -1053,7 +1173,7 @@ class PicklingProtocol2TestCase(unittest.TestCase):
         """
         instance = PicklableNamedTuple(('a', 'b'), (1, 2))
         newinstance = PicklableNamedTuple.__new__(PicklableNamedTuple,
-                                                 *(instance.__getnewargs__()))
+                                                  *(instance.__getnewargs__()))
         self.assertEqual(instance, newinstance)
 
     def test_getnewargs_priority(self):
@@ -1129,7 +1249,7 @@ class PicklingProtocol2TestCase(unittest.TestCase):
     def test_handles_cyclical_objects(self):
         child = PickleProtocol2Thing(None)
         instance = PickleProtocol2Thing(child, child)
-        child.args = (instance,) # create a cycle
+        child.args = (instance,)  # create a cycle
         # TODO we do not properly restore references inside of lists.
         # Change the above tuple into a list to show the breakage.
 
@@ -1162,7 +1282,7 @@ class PicklingProtocol2TestCase(unittest.TestCase):
     def test_handles_cyclical_objects_in_lists(self):
         child = PickleProtocol2ChildThing(None)
         instance = PickleProtocol2ChildThing([child, child])
-        child.child = instance # create a cycle
+        child.child = instance  # create a cycle
 
         encoded = jsonpickle.encode(instance)
         decoded = jsonpickle.decode(encoded)
@@ -1173,12 +1293,13 @@ class PicklingProtocol2TestCase(unittest.TestCase):
 
 def suite():
     suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(JSONPickleTestCase))
     suite.addTest(unittest.makeSuite(PicklingTestCase))
     suite.addTest(unittest.makeSuite(PicklingProtocol2TestCase))
-    suite.addTest(unittest.makeSuite(JSONPickleTestCase))
+    suite.addTest(unittest.makeSuite(PicklingProtocol4TestCase))
+    suite.addTest(doctest.DocTestSuite(jsonpickle))
     suite.addTest(doctest.DocTestSuite(jsonpickle.pickler))
     suite.addTest(doctest.DocTestSuite(jsonpickle.unpickler))
-    suite.addTest(doctest.DocTestSuite(jsonpickle))
     return suite
 
 if __name__ == '__main__':
