@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 import collections
 import decimal
 import re
@@ -7,9 +9,7 @@ import unittest
 import jsonpickle
 from jsonpickle import handlers
 from jsonpickle import tags
-from jsonpickle.compat import queue
-from jsonpickle.compat import set
-from jsonpickle.compat import unicode
+from jsonpickle.compat import queue, set, unicode, bytes, PY2
 
 
 class Thing(object):
@@ -30,6 +30,7 @@ class ListSubclass(list):
 class BrokenReprThing(Thing):
     def __repr__(self):
         raise Exception('%s has a broken repr' % self.name)
+
     def __str__(self):
         return '<BrokenReprThing "%s">' % self.name
 
@@ -104,11 +105,11 @@ class ThingWithFunctionRefs(object):
     def __init__(self):
         self.fn = func
 
+
 def func(x):
     return x
 
 
-
 class ThingWithQueue(object):
 
     def __init__(self):
@@ -118,7 +119,6 @@ class ThingWithQueue(object):
         self.childref_2 = self.child_2
 
 
-
 class ThingWithSlots(object):
 
     __slots__ = ('a', 'b')
@@ -153,6 +153,41 @@ class ThingWithStringSlots(object):
         self.ab = a + b
 
 
+class ThingWithSelfAsDefaultFactory(collections.defaultdict):
+    """defaultdict subclass that uses itself as its default factory"""
+
+    def __init__(self):
+        self.default_factory = self
+
+    def __call__(self):
+        return self.__class__()
+
+
+class ThingWithClassAsDefaultFactory(collections.defaultdict):
+    """defaultdict subclass that uses its class as its default factory"""
+
+    def __init__(self):
+        self.default_factory = self.__class__
+
+    def __call__(self):
+        return self.__class__()
+
+
+try:
+    import enum
+
+    class IntEnumTest(enum.IntEnum):
+        X = 1
+        Y = 2
+
+    class StringEnumTest(enum.Enum):
+        A = 'a'
+        B = 'b'
+except ImportError:
+    IntEnumTest = None
+    StringEnumTest = None
+
+
 class AdvancedObjectsTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -193,10 +228,53 @@ class AdvancedObjectsTestCase(unittest.TestCase):
         self.assertEqual(newdefaultdict[0], 'zero')
         self.assertEqual(type(newdefaultdict[1]), collections.defaultdict)
         self.assertEqual(newdefaultdict[1][0], 'zero')
-        self.assertEqual(newdefaultdict[1][1], {}) # inner defaultdict
-        self.assertEqual(newdefaultdict[2][0], 0) # outer defaultdict
+        self.assertEqual(newdefaultdict[1][1], {})  # inner defaultdict
+        self.assertEqual(newdefaultdict[2][0], 0)  # outer defaultdict
         self.assertEqual(type(newdefaultdict[3]), collections.defaultdict)
-        self.assertEqual(newdefaultdict[3].default_factory, int) # outer-most defaultdict
+        self.assertEqual(newdefaultdict[3].default_factory, int)  # outer-most defaultdict
+
+    def test_defaultdict_subclass_with_self_as_default_factory(self):
+        cls = ThingWithSelfAsDefaultFactory
+        tree = cls()
+        newtree = self._test_defaultdict_tree(tree, cls)
+        self.assertEqual(type(newtree['A'].default_factory), cls)
+        self.assertTrue(newtree.default_factory is newtree)
+        self.assertTrue(newtree['A'].default_factory is newtree['A'])
+        self.assertTrue(newtree['Z'].default_factory is newtree['Z'])
+
+    def test_defaultdict_subclass_with_class_as_default_factory(self):
+        cls = ThingWithClassAsDefaultFactory
+        tree = cls()
+        newtree = self._test_defaultdict_tree(tree, cls)
+        self.assertTrue(newtree.default_factory is cls)
+        self.assertTrue(newtree['A'].default_factory is cls)
+        self.assertTrue(newtree['Z'].default_factory is cls)
+
+    def _test_defaultdict_tree(self, tree, cls):
+        tree['A']['B'] = 1
+        tree['A']['C'] = 2
+        # roundtrip
+        encoded = jsonpickle.encode(tree)
+        newtree = jsonpickle.decode(encoded)
+        # make sure we didn't lose anything
+        self.assertEqual(type(newtree), cls)
+        self.assertEqual(type(newtree['A']), cls)
+        self.assertEqual(newtree['A']['B'], 1)
+        self.assertEqual(newtree['A']['C'], 2)
+        # ensure that the resulting default_factory is callable and creates
+        # a new instance of cls.
+        self.assertEqual(type(newtree['A'].default_factory()), cls)
+        # we've never seen 'D' before so the reconstructed defaultdict tree
+        # should create an instance of cls.
+        self.assertEqual(type(newtree['A']['D']), cls)
+        # ensure that proxies do not escape into user code
+        self.assertNotEqual(type(newtree.default_factory),
+                            jsonpickle.unpickler._Proxy)
+        self.assertNotEqual(type(newtree['A'].default_factory),
+                            jsonpickle.unpickler._Proxy)
+        self.assertNotEqual(type(newtree['A']['Z'].default_factory),
+                            jsonpickle.unpickler._Proxy)
+        return newtree
 
     def test_deque_roundtrip(self):
         """Make sure we can handle collections.deque"""
@@ -401,7 +479,7 @@ class AdvancedObjectsTestCase(unittest.TestCase):
         broken __repr__ implementations.
         """
         br = BrokenReprThing('test')
-        obj = { br: True }
+        obj = {br: True}
         pickler = jsonpickle.pickler.Pickler()
         flattened = pickler.flatten(obj)
         self.assertTrue('<BrokenReprThing "test">' in flattened)
@@ -464,13 +542,8 @@ class AdvancedObjectsTestCase(unittest.TestCase):
         obj['key1'] = 1
 
         flattened = self.pickler.flatten(obj)
-        self.assertEqual({'key1': 1,
-                          tags.OBJECT:
-                            'object_test.DictSubclass'
-                         },
-                         flattened)
-        self.assertEqual(flattened[tags.OBJECT],
-                         'object_test.DictSubclass')
+        self.assertEqual({'key1': 1, tags.OBJECT: 'object_test.DictSubclass'}, flattened)
+        self.assertEqual(flattened[tags.OBJECT], 'object_test.DictSubclass')
 
         inflated = self.unpickler.restore(flattened)
         self.assertEqual(1, inflated['key1'])
@@ -597,6 +670,60 @@ class AdvancedObjectsTestCase(unittest.TestCase):
         self.assertEqual('fat', match.group(1))
         self.assertEqual('cat', match.group(2))
 
+    def test_base_object_roundrip(self):
+        roundtrip = self.unpickler.restore(self.pickler.flatten(object()))
+        self.assertEqual(type(roundtrip), object)
+
+    def test_enum34(self):
+        if IntEnumTest is None or StringEnumTest is None:
+            return self.skipTest('enum module is not importable')
+
+        roundtrip = lambda obj: self.unpickler.restore(self.pickler.flatten(obj))
+
+        self.assertTrue(roundtrip(IntEnumTest.X) is IntEnumTest.X)
+        self.assertTrue(roundtrip(IntEnumTest) is IntEnumTest)
+
+        self.assertTrue(roundtrip(StringEnumTest.A) is StringEnumTest.A)
+        self.assertTrue(roundtrip(StringEnumTest) is StringEnumTest)
+
+    def test_bytes_unicode(self):
+        b1 = b'foo'
+        b2 = b'foo\xff'
+        u1 = u'foo'
+
+        # unicode strings get encoded/decoded as is
+        encoded = self.pickler.flatten(u1)
+        self.assertTrue(encoded == u1)
+        self.assertTrue(type(encoded) is unicode)
+        decoded = self.unpickler.restore(encoded)
+        self.assertTrue(decoded == u1)
+        self.assertTrue(type(decoded) is unicode)
+
+        # bytestrings are wrapped in PY3 but in PY2 we try to decode first
+        encoded = self.pickler.flatten(b1)
+        if PY2:
+            self.assertTrue(encoded == u1)
+            self.assertTrue(type(encoded) is unicode)
+        else:
+            self.assertTrue(encoded != u1)
+            self.assertTrue(encoded == {tags.BYTES: 'foo'})
+            self.assertTrue(type(encoded[tags.BYTES]) is unicode)
+        decoded = self.unpickler.restore(encoded)
+        self.assertTrue(decoded == b1)
+        if PY2:
+            self.assertTrue(type(decoded) is unicode)
+        else:
+            self.assertTrue(type(decoded) is bytes)
+
+        # bytestrings that we can't decode to UTF-8 will always be wrapped
+        encoded = self.pickler.flatten(b2)
+        self.assertTrue(encoded != b2)
+        self.assertTrue(encoded == {tags.BYTES: 'foo=FF'})
+        self.assertTrue(type(encoded[tags.BYTES]) is unicode)
+        decoded = self.unpickler.restore(encoded)
+        self.assertTrue(decoded == b2)
+        self.assertTrue(type(decoded) is bytes)
+
 
 # Test classes for ExternalHandlerTestCase
 class Mixin(object):
